function [samplePoints,samplingTime,rejectionRate] = looplessACHRBsamplerParallel(model,numSamples,workerIdx,method,numDiscarted,stepsPerPoint)
% Performs Modified Hit-And-Run for sampling the loopless space
% Inputs:  model sampling structure, numSamples, workerIdx, numDiscarted (optional),
%          stepsPerPoint (optional)
% Outputs: samplePoints, samplingTime, rejectionRate
% Updates:
%          15/03/16 Determination of active loop laws using sparse Nint
%%%%%%%%%%%%%%%%%%%%%% Pedro Saa UQ 2015 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check inputs
if nargin<3; disp('Not enough input parameters'); return; end;

% Set-up intial parameters
n               = model.numRxns;
%activeLaws      = findActiveLoopLaws(model);  % Find potentially activeloop laws topollogically (not used anymore. sparsifyNint() used instead)

% Hit-And-Run parameters set-up
if nargin<4
    method        = 'll_ACHRB';   % Sampler
    numDiscarted  = 1e3;          % Discarted points
    stepsPerPoint = 10;           % Steps per point
elseif nargin<5
    numDiscarted  = 1e3;
    stepsPerPoint = 10;
elseif nargin<6
    stepsPerPoint = 10;
end

% Define algorithmic method
if strcmp(method,'ACHR') || (strcmp(method,'ll_ACHRB') && isempty(model.Nint)) % The last condition applies only if Nint is null
    loopless = 0;
else % method is ll_ACHRB
    loopless = 1;                               % Perform loop checking if method is ll_ACHRB
    nonZeroElements = sum((model.Nint~=0),2);
    loopMatrix      = sign(model.Nint);
end

% Set-up parameters for each sampler: ACHR and ll-ACHR/ACHRB
numWarmUps  = size(model.warmUp,2);
centerPoint = model.centroid;
prevPoint   = centerPoint;

% Allocating memory for the next points
samplePoints  = zeros(n,numSamples);
rejectionRate = zeros(numSamples*stepsPerPoint+numDiscarted,1);

% Direction tolerance
uTol = 1e-10;

% Minimum allowed distance to closest bound
maxMinTol = 1e-10;

% Maximum allowed discrepancy with equality constraints
dTol  = 1e-9;

% Main loop of the Hit-And-Run
counter       = 0;
sampleCount   = 0;
totalCount    = 0;
rejectedCount = 0;
if workerIdx==1
    fprintf('Sample point \tTime \t Time left \t Rejection rate \t Worker\n');
end
t0 = cputime;
while sampleCount<numSamples
    % Update count
    totalCount = totalCount+1;
    
    % Print step information
    if totalCount>1
        timeElapsed = (cputime-t0);
        timePerStep = timeElapsed/sampleCount;
        if ~mod(totalCount,1e3*stepsPerPoint) && counter>0 && workerIdx==1
            fprintf('%d\t%8.2f\t%8.2f\t%8.2f\t%8.2d\n',sampleCount,timeElapsed/60,(numSamples-sampleCount)*timePerStep/60,...
                rejectionRate(counter),workerIdx);
        end
    end
    
    % Return if the dynamics is frozen
    if ~isempty(rejectionRate) && counter>0 && rejectionRate(counter)>.9999 && totalCount>1e6
        disp('Hit-And-Run dynamics is frozen!')
        break;
    end
    
    % Sample random direction
    u = (model.warmUp(:,randi(numWarmUps))-centerPoint);    
    u = u/norm(u);
    
    % Figure out the distances to upper and lower bounds
    distUb = (model.ub(1:n)-prevPoint);
    distLb = (prevPoint-model.lb(1:n));
    
    % Figure out positive and negative directions
    posDirn = (u>uTol);
    negDirn = (u<-uTol);
    
    % Figure out all the possible maximum and minimum step sizes
    maxStepTemp = distUb./u;
    minStepTemp = -distLb./u;   
    maxStepVec  = [maxStepTemp(posDirn);minStepTemp(negDirn)];
    minStepVec  = [minStepTemp(posDirn);maxStepTemp(negDirn)];
    
    % Figure out the true max & min step sizes
    maxStep = min(maxStepVec);
    minStep = max(minStepVec);
    
    % Find new direction if we're getting too close to a constraint
    if (abs(minStep)<maxMinTol && abs(maxStep)<maxMinTol) || (minStep>maxStep) || (maxStep<0) || (minStep>0)
        rejectedCount = rejectedCount+1;
        continue;
    end
    
    % Pick a random step distance
    stepDist = rand(1)*(maxStep-minStep)+minStep;
    
    % Make move and check whether it is feasible
    nextPoint = prevPoint+stepDist*u;
    
    % Reproject the current point into the null space if
    if max(abs(model.S*nextPoint))>dTol
        nextPoint = model.N*(model.N'*nextPoint);
    end
    
    % Bring points to the boundary if they fall out
    if any((model.ub(1:n)-nextPoint)<0)
        overInd = (nextPoint>model.ub(1:n));
        nextPoint(overInd) = model.ub(overInd);
    end
    if any((nextPoint-model.lb(1:n))<0)
        underInd = (nextPoint<model.lb(1:n));
        nextPoint(underInd) = model.lb(underInd);
    end
    
    % Check loopless condition topologically
    if loopless
        looplessModel = 0;        
        checkPattern  = diag(sign(nextPoint(model.internal)));
        if all(abs(sum(loopMatrix*checkPattern,2))~=nonZeroElements)
            looplessModel = 1;
        end
    else
        looplessModel = 1;
    end
    if looplessModel
        % Save current point
        counter   = counter+1;
        prevPoint = nextPoint;
        rejectionRate(counter) = rejectedCount/totalCount;
        
        % Recalculate center point if the point is feasible
        centerPoint = ((numWarmUps+counter)*centerPoint+nextPoint)/(numWarmUps+counter+1);
        
        % Save sampled point
        if counter>numDiscarted && ~mod(counter,stepsPerPoint)
            sampleCount                 = sampleCount+1;
            samplePoints(:,sampleCount) = nextPoint;
        end
    else
        % Update rejected count
        rejectedCount = rejectedCount+1;
        
        % Perform loopless ACHRB
        while true
            totalCount = totalCount+1;
            
            % 'Shrink' the box according the accelerated HR on a Box
            if stepDist>0
                maxStep = stepDist;
            else
                minStep = stepDist;
            end
            
            % Check feasibility of step sizes
            if (abs(minStep)<maxMinTol && abs(maxStep)<maxMinTol) || (minStep>maxStep) || (maxStep<0) || (minStep>0)
                rejectedCount = rejectedCount+1;
                break;
            end
            
            % Get new step size
            stepDist = rand(1)*(maxStep-minStep)+minStep;
            
            % Make move and check whether it is feasible
            nextPoint = prevPoint+stepDist*u;
            
            % Reproject the current point into the null space if
            if max(abs(model.S*nextPoint))>dTol
                nextPoint = model.N*(model.N'*nextPoint);
            end
            
            % Bring points to the boundary if they fall out
            if any((model.ub(1:n)-nextPoint)<0)
                overInd = (nextPoint>model.ub(1:n));
                nextPoint(overInd) = model.ub(overInd);
            end
            if any((nextPoint-model.lb(1:n))<0)
                underInd = (nextPoint<model.lb(1:n));
                nextPoint(underInd) = model.lb(underInd);
            end
            
            % Check loopless condition topologically
            checkPattern  = diag(sign(nextPoint(model.internal)));
            if all(abs(sum(loopMatrix*checkPattern,2))~=nonZeroElements)
                % Save current point
                counter       = counter+1;
                prevPoint     = nextPoint;
                rejectionRate(counter) = rejectedCount/totalCount;
                
                % Recalculate center point if the point is feasible
                centerPoint = ((numWarmUps+counter)*centerPoint+nextPoint)/(numWarmUps+counter+1);
                
                % Save sampled point
                if counter>numDiscarted && ~mod(counter,stepsPerPoint)
                    sampleCount                 = sampleCount+1;
                    samplePoints(:,sampleCount) = nextPoint;
                end
                break;
            else
                % Update counters
                rejectedCount = rejectedCount+1;
            end
        end
    end
end
samplingTime = (cputime-t0)/60;