function model = looplessFVA(model,FVAtype,params)
% Performs ll-FVA or linear FVA
% Inputs:  model structure
% Outputs: model structure with new bounds and directionalities
%%%%%%%%%%%%%%%%%%%%%% Pedro Saa UQ 2015 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<2
    FVAtype = 'loopless';
    params.outputflag     = 0;
    params.OptimalityTol  = 1e-9;
    params.IntFeasTol     = 1e-9;
    params.FeasibilityTol = 1e-9;
    params.MIPGapAbs      = 1e-12;
    params.MIPGap         = 1e-12;    
elseif nargin<3
    params.outputflag     = 0;
    params.OptimalityTol  = 1e-9;
    params.IntFeasTol     = 1e-9;
    params.FeasibilityTol = 1e-9;
    params.MIPGapAbs      = 1e-12;
    params.MIPGap         = 1e-12;    
end
tol = 1e-8;

% Define and run FVA problem
f   = zeros(size(model.obj));
FVA = zeros(model.numRxns,2);

% Main loop
t0 = cputime;
for i = 1:model.numRxns
    model.obj(i) = 1;    
    
    % Minimization problem
    model.modelsense = 'min';    
    sol          = gurobi(model,params);
    FVA(i,1)     = sol.x(i)*(abs(sol.x(i))>tol);
    
    % Maximization problem
    model.modelsense = 'max';
    sol          = gurobi(model,params);
    FVA(i,2)     = sol.x(i)*(abs(sol.x(i))>tol);   
   
    % Restart objective
    model.obj    = f;
end

% Find zero reactions
zeroRxns = find(abs(FVA(:,1))+abs(FVA(:,2))==0);

if isempty(zeroRxns)
    % Reset objective function
    model.obj = f;
    
    % Redefine model boundaries
    model.lb(1:model.numRxns) = FVA(:,1);
    model.ub(1:model.numRxns) = FVA(:,2);

    % Re-format the problem
    if strcmp(FVAtype,'loopless')        
        model = looplessStructureMILP(model);
    elseif strcmp(FVAtype,'linear')
        model = linearStructure(model);
    end
else
    % Remove zero rxns
    try
        modelTemp.S = model.S;
    catch
        modelTemp.S = model.A;
    end
    modelTemp.S(:,zeroRxns) = [];
    modelTemp.c             = zeros(size(modelTemp.S,2),1);
    
    % Find orphan metabolites
    orphanMets = find(sum(full(abs(modelTemp.S)),2)==0);
    modelTemp.S(orphanMets,:) = [];
    
    % Re-define other quantities
    modelTemp.b                    = zeros(size(modelTemp.S,1),1);
    modelTemp.lb                   = FVA(:,1);
    modelTemp.lb(zeroRxns)         = [];
    modelTemp.ub                   = FVA(:,2);
    modelTemp.ub(zeroRxns)         = [];
    modelTemp.rxns                 = model.rxns;
    modelTemp.rxns(zeroRxns)       = [];
    modelTemp.rxnNames             = model.rxnNames;
    modelTemp.rxnNames(zeroRxns)   = [];
    modelTemp.mets                 = model.mets;
    modelTemp.mets(orphanMets)     = [];
    modelTemp.metNames             = model.metNames;
    modelTemp.metNames(orphanMets) = [];
    modelTemp.description          = model.description;

    % Re-format the problem
    if strcmp(FVAtype,'loopless')
        model = looplessStructureMILP(modelTemp);
    elseif strcmp(FVAtype,'linear')
        model = linearStructure(modelTemp);
    end
end

% Assign reversibilities to thermodynamically allowable rxns
model.rev = (model.lb(1:model.numRxns)<0).*(model.ub(1:model.numRxns)>0);

% Save preprocessing time (min)
model.prepTime = (cputime-t0)/60;
disp(['Done! ',num2str(length(zeroRxns)),' rxns removed']);